@@ -322,17 +322,15 @@ def adaptive_format(num: numbers.Number, prev_value: Union[numbers.Number, Itera
                 return '{0:,}'.format(num)
             elif 1.5 >= num >= 0.001 or -0.001 >= num >= -1.5 or num == 0:
                 return '{0:.3%}'.format(num)
-            elif 10000<num  or num <= -10000:
-                return '{0:{1}}'.format(num, '.3e')
-            else:
+            elif 10000>num  or num >= -10000:
                 return '{0:{1}}'.format(num, '.3f')
-
+            else:
+                return '{0:{1}}'.format(num, '.3e')
         elif value_type != 'loss':
-            if 10000<= num or  num <= -10000 :
-                return '{0:{1}}'.format(num, '.3e')
-            else:
+            if 10000 >= num >= 0.001 or -0.001 >= num >= -10000 or num == 0:
                 return '{0:{1}}'.format(num, '.3f')
-
+            else:
+                return '{0:{1}}'.format(num, '.3e')
 
         return '{0:{1}}'.format(num, format_string)
 
