@@ -868,7 +868,7 @@ class ModelBase(object):
                      'Loss: {0} | {1} | lr: {2:<10.3e} | epoch: {3}'.format(adaptive_format(loss_value, value_type='loss'), ', '.join(metric_strings),
                                                                             self.training_context['current_lr'],
                                                                             self.training_context['current_epoch']),
-                     name=self.name.ljust(self.training_context['max_name_length'] + 1, ' '))
+                     name=self.training_context['model_name'].ljust(self.training_context['max_name_length'] + 1, ' '))
         self.training_context['time_batch_progress'] = 0
 
         self.do_on_progress_end()
@@ -889,7 +889,7 @@ class ModelBase(object):
                      'Loss: {0}| {1} | lr: {2:<10.3e}'.format(
                          adaptive_format(self.epoch_loss_history.get_last('total_losses')[-1], self.epoch_loss_history.get_series('total_losses')[-1], value_type='loss',
                                          name='total_losses'), ', '.join(metric_strings), self.training_context['current_lr']),
-                     name=self.name.ljust(self.training_context['max_name_length'] + 1, ' '))
+                     name=self.training_context['model_name'].ljust(self.training_context['max_name_length'] + 1, ' '))
         self.training_context['time_epoch_progress'] = 0
 
         self.do_on_progress_end()
@@ -914,6 +914,9 @@ class ModelBase(object):
                 if self.training_context['current_epoch'] == 0:
                     # epoch is not the logical inteval for us to control the flow
                     self.training_context['steps'] = 0
+                    #on_epoch_start
+                    for callback in self.callbacks:
+                        callback.on_epoch_start(self.training_context)
 
                     self.training_context['grads_state'] = OrderedDict()
                     self.training_context['grads_state']['first_layer'] = []
@@ -1036,7 +1039,8 @@ class ModelBase(object):
         raise NotImplementedError
 
     def trigger_when(self, when='on_batch_end', frequency=None, unit='batch', action=None):
-
+        if 'epoch' in when:
+            unit='epoch'
         new_callbacks =LambdaCallback(when, frequency=frequency, unit=unit, action=action)
         self.with_callbacks(new_callbacks)
         return self
