@@ -24,7 +24,7 @@ elif get_backend()=='tensorflow':
 
 
 
-__all__ = ['AdjustLRCallback','ReduceLROnPlateau','reduce_lr_on_plateau','LambdaLR','lambda_lr','RandomCosineLR','random_cosine_lr','CosineLR','cosine_lr','OnceCycleLR','StepLR']
+__all__ = ['AdjustLRCallback','ReduceLROnPlateau','reduce_lr_on_plateau','LambdaLR','lambda_lr','RandomCosineLR','PolyLR','random_cosine_lr','CosineLR','cosine_lr','OnceCycleLR','StepLR']
 
 
 
@@ -218,6 +218,18 @@ class StepLR(AdjustLRCallbackBase):
             self.adjust_learning_rate(training_context, training_context['optimizer'].lr* self.gamma, verbose=True)
 
 
+class PolyLR(AdjustLRCallbackBase):
+    def __init__(self,max_lr=1e-3,  max_iter=10000):
+        super().__init__()
+        self.max_lr = max_lr
+        self.max_iter=max_iter
+    def on_batch_end(self, training_context):
+        current_step =training_context['steps']
+        lr = self.max_lr * (1 - (current_step/ self.max_iter)) * (1 - (current_step / self.max_iter))
+        if (lr < 1.0e-7):
+            lr = 1.0e-7
+        self.adjust_learning_rate(training_context, lr, verbose=False)
+
 
 
 class ReduceLROnPlateau(AdjustLRCallbackBase):
