@@ -36,6 +36,9 @@ try:
 except ImportError:
     from six.moves.urllib.request import urlretrieve
 
+__all__ = ['load_mnist', 'load_lfw', 'load_text', 'load_cifar', 'load_kaggle', 'load_birdsnap', 'load_examples_data', 'load_stanford_cars', 'load_folder_images']
+
+
 _session = get_session()
 _trident_dir = os.path.join(_session.trident_dir, 'datasets')
 _backend = get_backend()
@@ -400,7 +403,7 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
                 labels.extend([i] * len(class_imgs))
                 imgs.extend(class_imgs)
 
-            imagedata = ImageDataset(imgs, object_type=ObjectType.rgb)
+            imagedata = ImageDataset(imgs, object_type=object_type)
             print('extract {0} images...'.format(len(imagedata)))
             labelsdata = LabelDataset(labels,object_type=ObjectType.classification_label)
             labelsdata.binding_class_names(class_names)
@@ -652,7 +655,7 @@ def load_examples_data(dataset_name):
                                           object_type=ObjectType.gray)
 
         dataset.testdata = dataset_test.traindata
-
+        dataset.class_names['zh-cn'] = dataset.class_names['en-us']
         return dataset
 
     elif dataset_name == 'examples_animals':
@@ -1163,4 +1166,4 @@ def load_examples_data(dataset_name):
         return provider
     #,sample_filter=lambda x:x[0][-1]==3
     else:
-        return None
\ No newline at end of file
+        return None
