@@ -191,10 +191,11 @@ class ImageDataProvider(object):
         if len(self.reverse_image_transform_funcs) == 0:
             return reverse_image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
+            img_data = reverse_image_backend_adaption(img_data)
             # if img_data.ndim>=2:
             for fc in self.reverse_image_transform_funcs:
                 img_data = fc(img_data)
-            # img_data = reverse_image_backend_adaption(img_data)
+
 
         return img_data
 
