@@ -442,22 +442,15 @@ class OneOf(Transform):
     """
 
     def __init__(
-            self, transforms=[], batch_compose=False, shuffle_indices=None
+            self, transforms=[], batch_compose=False
     ):
         super().__init__()
         self.transforms = transforms
-        self.random_index=0
-        #self._set_order()
+        self.shuffle_indices=list(range(len(self.transforms)))
+        self.random_index=random.choice( self.shuffle_indices)
 
-        if batch_compose and shuffle_indices is not None:
-            raise ValueError(
-                "Do not support shuffle when apply transforms along the whole batch"
-            )
         self.batch_compose = batch_compose
 
-        if shuffle_indices is not None:
-            shuffle_indices = [tuple(x - 1 for x in idx) for idx in shuffle_indices]
-        self.shuffle_indices = shuffle_indices
 
     def apply_batch(self,inputs: Sequence[Tuple],spec:Optional[TensorSpec]=None):
         if not isinstance(inputs, OrderedDict):
