@@ -12,7 +12,7 @@ import numpy as np
 from skimage.filters import threshold_minimum, threshold_local, threshold_isodata, threshold_yen
 
 from trident.backend.common import *
-from trident.backend.tensorspec import ObjectType
+from trident.backend.tensorspec import ObjectType, get_signature
 from trident.backend.tensorspec import TensorSpec
 
 if get_backend() == 'pytorch':
@@ -23,66 +23,88 @@ from trident.data.transform import VisionTransform
 
 __all__ = ['Resize', 'ShortestEdgeResize', 'Rescale', 'RandomCrop', 'RandomRescaleCrop', 'RandomCenterCrop', 'RandomMultiScaleImage', 'RandomTransform', 'RandomTransformAffine',
            'AdjustBrightness', 'AdjustContrast', 'AdjustSaturation', 'AddNoise', 'AdjustHue', 'RandomAdjustHue', 'RandomAdjustBrightness', 'RandomAdjustContrast',
-           'RandomAdjustSaturation','AutoLevel',
+           'RandomAdjustSaturation','AutoLevel','RandomLighting','GrayMixRGB',
            'Normalize', 'Unnormalize', 'CLAHE', 'Lighting', 'HorizontalFlip', 'RandomMirror', 'AdjustGamma', 'RandomBlur', 'RandomAdjustGamma', 'Blur', 'InvertColor',
            'RandomInvertColor', 'GrayScale', 'RandomGrayScale','RandomGridMask','GridMask',
            'ImageDilation', 'ImageErosion', 'ErosionThenDilation', 'DilationThenErosion', 'AdaptiveBinarization', 'SaltPepperNoise', 'RandomErasing', 'ToRGB', 'ImageMosaic','DetectionMixup']
 
 
-def randomize_with_validate(valid_range=None, no_change_value=None, **kwargs):
+
+def randomize_with_validate(keep_prob=0.5,valid_range=None, no_change_value=None, **kwargs):
     def randomize_wrapper(cls):
         class Wrapper:
             def __init__(self, **kwargs):
+                self._args=None
+                self.rangs=None
+                self.other_args=None
+                self.keep_prob =keep_prob
                 self.valid_range = None
                 self.no_change_value = None
+
                 if isinstance(valid_range, (tuple, list)) and len(valid_range) == 2 and all([isinstance(t, numbers.Number) for t in valid_range]):
                     self.valid_range = valid_range
+                if 'scale' in kwargs and len(kwargs[ 'scale'])==2 and len([k for k,v in kwargs.items() if len(v)==2 ])==1 :
+                    self.valid_range=kwargs[ 'scale']
+                    kwargs.pop('scale')
+
                 if isinstance(no_change_value, numbers.Number):
                     self.no_change_value = float(no_change_value)
 
-                rangs = dict([(k.replace('_range', ''), random.uniform(*v)) for k, v in kwargs.items() if isinstance(v, tuple) and len(v) == 2])
-                other_rangs = dict([(k, v) for k, v in kwargs.items() if not (isinstance(v, tuple) and len(v) == 2)])
 
-                self.kwargs = kwargs
+                # argspec = inspect.getfullargspec(cls.__init__)
+                # _args = argspec.args
+                # _args.remove('self')
+                # _args.remove('name')
+                self._args = get_signature(cls.__init__,name=cls.__name__)
 
-                argspec = inspect.getfullargspec(cls.__init__)
-                _args = argspec.args
-                _args.remove('self')
-                _args.remove('name')
+                self.rangs = dict([(k.replace('_range', ''), v) for k, v in kwargs.items() if isinstance(v, tuple) and len(v) == 2])
+                for argname,arg in self._args.inputs.items():
+                    if argname not in ['self','name','keep_ratio'] and 'size' not in argname and 'shape' not in argname and argname not in self.rangs and isinstance(arg.default,numbers.Number):
+                        self.rangs[argname]=self.valid_range
+
+                rangs = dict([(k.replace('_range', ''), random.uniform(*v)) for k, v in self.rangs.items()])
+                other_args = dict([(k, v.default) for k, v in self._args.inputs.items() if k.replace('_range', '') not in self.rangs ])
+                self.other_args = other_args
 
-                for k, v in self.kwargs.items():
-                    if k in _args:
-                        _args.remove(k)
-                self._args = _args
                 if len(rangs.values()) == 0 and self.valid_range is not None:
-                    if len(_args) == 0 and len(self.kwargs) == 0:
+                    if len(self.rangs) == 0 and len(kwargs) == 0:
                         self.wrap = cls()
                     else:
-                        self.wrap = cls(random.uniform(*self.valid_range), **other_rangs)
+                        self.wrap = cls(random.uniform(*self.valid_range), **other_args)
                 else:
 
-                    self.wrap = cls(*list(rangs.values()), **other_rangs)
-                self.rn = 0
+                    self.wrap = cls(*list(rangs.values()), **other_args)
+                self.rn = random.random()
 
             def __call__(self, inputs: Union[Dict[TensorSpec, np.ndarray], np.ndarray], spec: TensorSpec = None, **kwargs):
-                rangs, other_rangs = self.set_random()
-                if len(rangs) > 0 and len(self.kwargs) > 0:
-                    for k, v in rangs:
-                        setattr(self.wrap, k, v)
-                else:
-                    if len(self._args) > 0:
-                        setattr(self.wrap, self._args[0], random.uniform(*self.valid_range))
+                keep_prob = kwargs.get('keep_prob', kwargs.get('keep_ratio', self.keep_prob))
+                self.keep_prob =keep_prob
+                self.rn=random.random()
+                if self.rn > keep_prob:
+                    rangs, other_args = self.set_random()
 
-                if self.rn % 5 > 0:
+                    for k,v in rangs.items():
+                        setattr(self.wrap, k, v)
+                    for k,v in other_args.items():
+                        setattr(self.wrap, k, v)
+
+                    # if len(rangs) > 0 and len(self.kwargs) > 0:
+                    #     for k, v in rangs:
+                    #         setattr(self.wrap, k, v)
+                    # else:
+                    #     if len(self._args) > 0:
+                    #         setattr(self.wrap, self._args[0], random.uniform(*self.valid_range))
                     return self.wrap(inputs, spec=spec, **kwargs)
                 else:
                     return inputs
 
             def set_random(self):
-                self.rn = random.randint(0, 10)
-                rangs = dict([(k.replace('_range', ''), random.uniform(*v)) for k, v in kwargs.items() if isinstance(v, tuple) and len(v) == 2])
-                other_rangs = dict([(k, v) for k, v in kwargs.items() if not (isinstance(v, tuple) and len(v) == 2)])
-                return rangs, other_rangs
+                self.rn = random.random()
+                #self.rn = random.randint(0, 10)
+
+                rangs = dict([(k.replace('_range', ''), random.uniform(*v)) for k, v in self.rangs.items() if isinstance(v, tuple) and len(v) == 2])
+                other_args = dict([(k, v) for k, v in self.other_args.items() if not (isinstance(v, tuple) and len(v) == 2)])
+                return rangs, other_args
 
         Wrapper.__name__ = Wrapper.__qualname__ = cls.__name__
         Wrapper.__doc__ = cls.__doc__
@@ -90,18 +112,19 @@ def randomize_with_validate(valid_range=None, no_change_value=None, **kwargs):
     return randomize_wrapper
 
 
-def randomize(keep_ratio=0.2, **kwargs):
+def randomize(keep_prob=0.5, **kwargs):
     def randomize_wrapper(cls):
         class Wrapper:
             def __init__(self, **kwargs):
                 self.kwargs = kwargs
                 self.wrap = cls()
-                self.keep_ratio = keep_ratio
+                self.keep_prob = keep_prob
                 self.rn = 0
 
             def __call__(self, inputs: Union[Dict[TensorSpec, np.ndarray], np.ndarray], spec: TensorSpec = None, **kwargs):
-                keep_ratio = kwargs.get('keep_ratio', self.keep_ratio)
-                if self.rn > keep_ratio:
+                keep_prob = kwargs.get('keep_prob', kwargs.get('keep_ratio', self.keep_prob))
+                self.keep_prob =keep_prob
+                if self.rn > keep_prob:
                     return self.wrap(inputs, spec=spec, **kwargs)
                 else:
                     return inputs
@@ -114,7 +137,6 @@ def randomize(keep_ratio=0.2, **kwargs):
         return Wrapper
     return randomize_wrapper
 
-
 class Resize(VisionTransform):
     r"""
     Resize the input data.
@@ -172,8 +194,8 @@ class Resize(VisionTransform):
         h, w, th, tw, pad_vert, pad_horz = self._shape_info
         if h == th and w == tw:
             return coords
-        coords[:, 0] = coords[:, 0] * (tw / w)
-        coords[:, 1] = coords[:, 1] * (th / h)
+        coords[:, 0] = np.round(coords[:, 0] * (float(tw) / w))
+        coords[:, 1] = np.round(coords[:, 1] * (float(th) / h))
         if not self.align_corner:
             coords[:, 0] += pad_horz // 2
             coords[:, 1] += pad_vert // 2
@@ -228,7 +250,7 @@ class Resize(VisionTransform):
 
 
 class ShortestEdgeResize(VisionTransform):
-    def __init__(self, output_size, keep_aspect=True, interpolation=cv2.INTER_LANCZOS4, name='short_edge_resize', **kwargs):
+    def __init__(self, output_size, keep_aspect=True, interpolation=cv2.INTER_LANCZOS4, name='short_edge_resize'):
         super().__init__(name)
         self.is_spatial = True
         self.output_size = output_size
@@ -609,7 +631,7 @@ class RandomTransformAffine(VisionTransform):
     :param order: the same with :class:`VisionTransform`.
     """
 
-    def __init__(self, rotation_range=15, zoom_range=0.02, shift_range=0.02, shear_range=0.2, random_flip=0.15,border_mode='random_color', interpolation=cv2.INTER_AREA,keep_ratio=0.5, name='transform_affine', **kwargs):
+    def __init__(self, rotation_range=15, zoom_range=0.02, shift_range=0.02, shear_range=0.2, random_flip=0.15,border_mode='random_color', interpolation=cv2.INTER_AREA,keep_prob=0.5, name='transform_affine', **kwargs):
         super().__init__(name)
         self.is_spatial = True
         self.output_size = None
@@ -624,7 +646,7 @@ class RandomTransformAffine(VisionTransform):
             self.border_mode ='random_color'
         else:
             self.border_mode=border_mode
-        self.keep_ratio=keep_ratio
+        self.keep_prob=keep_prob
 
     def apply(self, input: Tuple, spec: TensorSpec):
         return super().apply(input, spec)
@@ -639,7 +661,7 @@ class RandomTransformAffine(VisionTransform):
             self._shape_info = self._get_shape(image)
         mat_img, height, width,angle, is_flip ,rr,shear_factor= self._shape_info
 
-        if rr>self.keep_ratio:
+        if rr>self.keep_prob:
             image = np.clip(image.astype(np.float32), 0, 255)[:, :, :3]
             _borderMode=cv2.BORDER_CONSTANT
             _borderValue = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))
@@ -666,7 +688,7 @@ class RandomTransformAffine(VisionTransform):
 
     def _apply_coords(self, coords, spec: TensorSpec):
         mat_img, height, width,angle, is_flip,rr,shear_factor = self._shape_info
-        if rr>self.keep_ratio:
+        if rr>self.keep_prob:
 
             coords = coords.transpose([1, 0])
             coords = np.insert(coords, 2, 1, axis=0)
@@ -688,7 +710,7 @@ class RandomTransformAffine(VisionTransform):
     def _apply_mask(self, mask, spec: TensorSpec):
         mask = unpack_singleton(mask)
         mat_img, height, width,angle, is_flip,rr ,shear_factor= self._shape_info
-        if rr>self.keep_ratio:
+        if rr>self.keep_prob:
             mask_dtype = mask.dtype
             mask = mask.astype(np.float32)
 
@@ -840,7 +862,7 @@ class VerticalFlip(VisionTransform):
         return height, width
 
 
-@randomize(keep_ratio=0.8)
+@randomize(keep_prob=0.8)
 class RandomMirror(HorizontalFlip):
     pass
 
@@ -1190,6 +1212,36 @@ class RandomAdjustGamma(AdjustGamma):
     pass
 
 
+class GrayMixRGB(VisionTransform):
+
+    def __init__(self, keep_prob=0.5, name='gray_mix_rgb', **kwargs):
+        super().__init__(name)
+
+        self.keep_prob = keep_prob
+
+    def apply(self, input: Tuple, spec: TensorSpec):
+        return super().apply(input, spec)
+
+    def _apply_image(self, image, spec: TensorSpec):
+        if random.random() <= self.keep_prob:
+            return image
+
+        dtype = image.dtype
+        image = image.astype(np.float32)
+        gray = cv2.cvtColor(cv2.cvtColor(image, cv2.COLOR_RGB2GRAY), cv2.COLOR_GRAY2RGB)
+        gray0 = gray[:, :, 0]
+        min_rgb = image.mean(-1)
+        mask = np.expand_dims(np.greater(gray0, min_rgb), -1)
+        image = mask * gray + (1 - mask) * image
+
+        return image.clip(0, 255).astype(np.float32)
+
+    def _apply_coords(self, coords, spec: TensorSpec):
+        return coords
+
+    def _apply_mask(self, mask, spec: TensorSpec):
+        return mask
+
 class Blur(VisionTransform):
     def __init__(self, ksize=5, name='blur', **kwargs):
         super().__init__(name)
@@ -1262,7 +1314,6 @@ class GrayScale(VisionTransform):
     def _apply_mask(self, mask, spec: TensorSpec):
         return mask
 
-
 class ToRGB(VisionTransform):
     def __init__(self, name='to_rgb', **kwargs):
         super().__init__(name)
@@ -1292,12 +1343,12 @@ class RandomBlur(Blur):
     pass
 
 
-@randomize(keep_ratio=0.8)
+@randomize(keep_prob=0.8)
 class RandomInvertColor(InvertColor):
     pass
 
 
-@randomize(keep_ratio=0.8)
+@randomize(keep_prob=0.8)
 class RandomGrayScale(GrayScale):
     pass
 
@@ -1529,7 +1580,7 @@ class AdaptiveBinarization(VisionTransform):
 
 class Lighting(VisionTransform):
 
-    def __init__(self, scale=0, name='lighting', **kwargs):
+    def __init__(self, scale=0.0, name='lighting', **kwargs):
         super().__init__(name)
         if scale < 0:
             raise ValueError("lighting scale should be non-negative")
@@ -1551,10 +1602,10 @@ class Lighting(VisionTransform):
             return image
 
         dtype = image.dtype
-        image = image.astype(np.float32)
+        image = (image/255.0).astype(np.float32)
         alpha = np.random.normal(scale=self.scale, size=3)
         image = image + self.eigvec.dot(alpha * self.eigval)
-        return image.clip(0, 255).astype(dtype)
+        return (image*255).clip(0, 255).astype(dtype)
 
     def _apply_coords(self, coords, spec: TensorSpec):
         return coords
@@ -1562,6 +1613,11 @@ class Lighting(VisionTransform):
     def _apply_mask(self, mask, spec: TensorSpec):
         return mask
 
+@randomize_with_validate(valid_range=(0, 3.), no_change_value=0.)
+class RandomLighting(Lighting):
+    pass
+
+
 
 class CLAHE(VisionTransform):
 
@@ -1842,38 +1898,49 @@ class RandomGridMask(VisionTransform):
         gain (float): The constant multiplier.
     """
 
-    def __init__(self, output_size=None,d1=None, d2=None, rotate_range=(0.2,1), ratio = 0.2, mode=0, keep_prob=0.5, name='gridmask', **kwargs):
+    def __init__(self, output_size=None, d1=None, d2=None, max_d1=None, rotate_range=(0.2, 1), ratio=0.2, mode=0, keep_prob=0.5, name='gridmask', **kwargs):
         super().__init__(name)
-        self.output_size=output_size
+        self.output_size = output_size
         self.d1 = d1
-        self.d2=d2
+        self.d2 = d2
+        self.max_d1 = max_d1
         self.rotate_range = rotate_range
         self.ratio = ratio
         self.keep_prob = keep_prob
         self.mode = mode
-
+        self.current_mask = None
 
     def set_prob(self, epoch, max_epoch):
         self.prob = self.keep_prob * min(1, epoch / max_epoch)
 
-    def get_grid(self,image):
-        h,w=image.shape[:2]
+    def get_grid(self, image):
+        h, w = image.shape[:2]
         if self.output_size is None:
-            self.output_size=(h,w)
+            self.output_size = (h, w)
         hh = math.ceil((math.sqrt(h * h + w * w)))
-
+        d2 = self.d2
+        d1 = None
         if self.d2 is None:
-            self.d2=maximum(h,w)
+            d2 = minimum(h, w)
+        elif isinstance(self.d2, numbers.Number):
+            pass
+        elif isinstance(self.d2, (tuple, list)):
+            d2 = random.choice(list(self.d2))
+
         if self.d1 is None:
-            divisors=get_divisors(maximum(h,w))
+            divisors = get_divisors(minimum(h, w))
 
-            divisors=list(sorted(set(divisors)))
-            divisors=[d for d in divisors if d <=maximum(h,w)/2.0]
-            if maximum(h,w) in divisors:
-                divisors.remove(maximum(h,w))
-            self.d1 =random.choice(divisors)
+            divisors = list(sorted(set(divisors)))
+            divisors = [d for d in divisors if d <= minimum(h, w) / 2.0]
+            if self.max_d1 is not None:
+                divisors = [d for d in divisors if d <= self.max_d1]
+            if minimum(h, w) in divisors:
+                divisors.remove(minimum(h, w))
+            d1 = random.choice(divisors)
+        elif isinstance(self.d1, (tuple, list)):
+            d1 = random.choice(list(self.d1))
 
-        d = np.random.randint(self.d1, self.d2)
+        d = np.random.randint(d1, d2)
         # d = self.d
 
         # maybe use ceil? but i guess no big difference
@@ -1894,16 +1961,17 @@ class RandomGridMask(VisionTransform):
             s = max(min(s, hh), 0)
             t = max(min(t, hh), 0)
             mask[:, s:t] *= 0
-        r = np.random.randint(self.rotate_range[0],self.rotate_range[1],)
+        r = np.random.randint(self.rotate_range[0], self.rotate_range[1], )
         mask = Image.fromarray(np.uint8(mask))
         mask = mask.rotate(r)
         mask = np.asarray(mask)
         mask = mask[(hh - h) // 2:(hh - h) // 2 + h, (hh - w) // 2:(hh - w) // 2 + w]
 
-
         if self.mode == 1:
             mask = 1 - mask
-        image = image * np.expand_dims(mask,-1)
+        # 暫存mask以處理bbox
+        self.current_mask = mask.copy()
+        image = image * np.expand_dims(mask, -1)
 
         return image
 
@@ -1913,43 +1981,64 @@ class RandomGridMask(VisionTransform):
     def _apply_image(self, image, spec: TensorSpec):
         if self._shape_info is None:
             self._shape_info = self._get_shape(image)
-        h, w,rr= self._shape_info
-
-
-        if rr > self.keep_prob:
-            return self.get_grid(image)
-        else:
+        h, w, rr, keep = self._shape_info
+        height, width = self.output_size
+        if keep:
             return image
+        else:
+            return self.get_grid(image)
+
+    def _apply_boxes(self, boxes, spec: TensorSpec):
+        h, w, rr, keep = self._shape_info
+        if keep:
+            return boxes
+        elif boxes is None:
+            return boxes
+        else:
+            keep_boxes = []
+            location = boxes[:, :4]
+            class_info = boxes[:, 4:5] if boxes.shape[-1] > 4 else None
+            keypoints = boxes[:, 5:] if boxes.shape[-1] > 5 else None
+            # 如果剛好格線遮蔽了小框，則排除小框定義
+            boxes[:,2] = np.clip(boxes[:,2], 0, w)
+            boxes[:,3] = np.clip(boxes[:,3], 0, h)
+            for i in range(len(boxes)):
+                box=np.round(location[i]).astype(np.int32)
+                if self.current_mask[box[1]:box[3] + 1, box[0]:box[2] + 1].sum() == 0:
+                    pass
+                else:
+                    keep_boxes.append(boxes[i])
+            if len(keep_boxes)>1:
+                return np.stack(keep_boxes, 0)
+            elif len(keep_boxes)==1:
+                return np.array(keep_boxes)
+            else:
+                return None
+
+
 
     def _apply_coords(self, coords, spec: TensorSpec):
-        h, w,rr= self._shape_info
+        h, w, rr, keep = self._shape_info
         return coords
 
     def _apply_mask(self, mask, spec: TensorSpec):
-        h, w,rr = self._shape_info
+        h, w, rr, keep = self._shape_info
+        height, width = self.output_size
         return mask
 
     def _get_shape(self, image):
         h, w = image.shape[:2]
-        rr =random.random()
+
+        rr = random.random()
+        keep = rr < self.keep_prob
         if self.output_size is None:
             self.output_size = (h, w)
-        return h, w,rr
+        return h, w, rr, keep
+
+
 
 
 class DetectionMixup(VisionTransform):
-    """Perform gamma correction on an image.
-        Also known as Power Law Transform. Intensities in RGB mode are adjusted
-        based on the following equation:
-        I_out = 255 * gain * ((I_in / 255) ** gamma)
-        See https://en.wikipedia.org/wiki/Gamma_correction for more details.
-    Args:
-        gamma (float): Non negative real number. gamma larger than 1 make the
-            shadows darker, while gamma smaller than 1 make dark regions
-            lighter.
-        gain (float): The constant multiplier.
-    """
-
     def __init__(self, output_size=None, keep_prob=0.8, name='detection_mixup', **kwargs):
         super().__init__(name)
         self.alpha=1
@@ -1985,9 +2074,10 @@ class DetectionMixup(VisionTransform):
         keep, other_idx, lam = self._shape_info
         if keep:
             return boxes
+
         height, width = self.output_size
-        box1 = self.memory_cache[other_idx][spec].copy()
-        box0=boxes.copy()
+        box1 = self.memory_cache[other_idx][spec].copy()  if  self.memory_cache[other_idx][spec] is not None else None
+        box0=boxes.copy() if boxes is not None else None
         if boxes is None or len(boxes)==0 or np.array_equal(np.unique(boxes),-1*np.ones(1)):
             box0=None
         if box1 is None or len(box1)==0 or np.array_equal(np.unique(box1),-1*np.ones(1)):
@@ -2025,11 +2115,7 @@ class DetectionMixup(VisionTransform):
             trans_boxes = np.concatenate(merge_boxes, axis=0)
             return trans_boxes
         else:
-            return boxes
-
-
-
-
+            return None
 
     def _apply_mask(self, mask, spec: TensorSpec):
          return mask
@@ -2107,84 +2193,123 @@ class ImageMosaic(VisionTransform):
         if keep:
             return boxes
         height, width = self.output_size
-        box0 = boxes.copy()
-        location0 = box0[:, :4].reshape(box0.shape[0], 2, 2)
-        location0[:, :, 0] = np.clip(location0[:, :, 0], 0, center[1])
-        location0[:, :, 1] = np.clip(location0[:, :, 1], 0, center[0])
-
-        class_info0 = box0[:, 4:5] if box0.shape[-1] > 4 else None
-        keypoints0 = None
-        if box0.shape[-1] > 5:
-            keypoints0 = box0[:, 5:].reshape(box0.shape[0], -1, 2)
-            invisible_keypoints0 = np.less_equal(keypoints0, 0).astype(np.bool)
-            keypoints0[:, :, 0] = np.clip(keypoints0[:, :, 0], 0, center[1])
-            keypoints0[:, :, 1] = np.clip(keypoints0[:, :, 1], 0, center[0])
-            keypoints0[invisible_keypoints0] = -1
-        else:
+        concate_list_all = []
+        box0 = boxes.copy() if boxes is not None else None
+        if box0 is None or len(box0) == 0:
             pass
-
-        box0 = np.concatenate([location0.reshape((box0.shape[0], 4)), class_info0, keypoints0.reshape((box0.shape[0], -1))], axis=-1)
-
-        box1 = self.memory_cache[other_idxes[0]][spec].copy()
-        location1 = box1[:, :4].reshape(box1.shape[0], 2, 2)
-        location1[:, :, 0] = np.clip(location1[:, :, 0] - img1_offsetx + center[1], center[1], width)
-        location1[:, :, 1] = np.clip(location1[:, :, 1] - img1_offsety, 0, center[0])
-
-        class_info1 = box1[:, 4:5] if box1.shape[-1] > 4 else None
-        keypoints1 = None
-        if box1.shape[-1] > 5:
-            keypoints1 = box1[:, 5:].reshape(box1.shape[0], -1, 2)
-            invisible_keypoints1 = np.less_equal(keypoints1, 0).astype(np.bool)
-            keypoints1[:, :, 0] = np.clip(keypoints1[:, :, 0] - img1_offsetx + center[1], center[1], width)
-            keypoints1[:, :, 1] = np.clip(keypoints1[:, :, 1] - img1_offsety, 0, center[0])
-            keypoints1[invisible_keypoints1] = -1
-
         else:
+            location0 = box0[:, :4].reshape(box0.shape[0], 2, 2)
+            location0[:, :, 0] = np.clip(location0[:, :, 0], 0, center[1])
+            location0[:, :, 1] = np.clip(location0[:, :, 1], 0, center[0])
+
+            class_info0 = box0[:, 4:5] if box0.shape[-1] > 4 else None
+            keypoints0 = box0[:, 5:] if box0.shape[-1] > 5 else None
+            if box0.shape[-1] > 5:
+                keypoints0 = box0[:, 5:].reshape(box0.shape[0], -1, 2)
+                invisible_keypoints0 = np.less_equal(keypoints0, 0).astype(np.bool)
+                keypoints0[:, :, 0] = np.clip(keypoints0[:, :, 0], 0, center[1])
+                keypoints0[:, :, 1] = np.clip(keypoints0[:, :, 1], 0, center[0])
+                keypoints0[invisible_keypoints0] = -1
+            else:
+                pass
+            concate_list0 = None
+            if keypoints0 is None:
+                # print('concate_list0',location0.reshape((box0.shape[0], 4)).shape,class_info0.shape)
+                concate_list0 = [location0.reshape((box0.shape[0], 4)), class_info0]
+            else:
+                concate_list0 = [location0.reshape((box0.shape[0], 4)), class_info0, keypoints0.reshape((box0.shape[0], -1))]
+
+            box0 = np.concatenate(concate_list0, axis=-1)
+            concate_list_all.append(box0)
+
+        box1 =self.memory_cache[other_idxes[0]][spec].copy() if  self.memory_cache[other_idxes[0]][spec] is not None else None
+        if box1 is None or len(box1) == 0:
             pass
-
-        box1 = np.concatenate([location1.reshape((box1.shape[0], 4)), class_info1, keypoints1.reshape((box1.shape[0], -1))], axis=-1)
-
-        box2 = self.memory_cache[other_idxes[1]][spec].copy()
-        location2 = box2[:, :4].reshape(box2.shape[0], 2, 2)
-        location2[:, :, 0] = np.clip(location2[:, :, 0] - img2_offsetx, 0, center[1])
-        location2[:, :, 1] = np.clip(location2[:, :, 1] - img2_offsety + center[0], center[0], height)
-
-        class_info2 = box2[:, 4:5] if box2.shape[-1] > 4 else None
-        keypoints2 = None
-        if box2.shape[-1] > 5:
-            keypoints2 = box2[:, 5:].reshape(box2.shape[0], -1, 2)
-            invisible_keypoints2 = np.less_equal(keypoints2, 0).astype(np.bool)
-            keypoints2[:, :, 0] = np.clip(keypoints2[:, :, 0] - img2_offsetx, 0, center[1])
-            keypoints2[:, :, 1] = np.clip(keypoints2[:, :, 1] - img2_offsety + center[0], center[0], height)
-            keypoints2[invisible_keypoints2] = -1
-
-
         else:
+            location1 = box1[:, :4].reshape(box1.shape[0], 2, 2)
+            location1[:, :, 0] = np.clip(location1[:, :, 0] - img1_offsetx + center[1], center[1], width)
+            location1[:, :, 1] = np.clip(location1[:, :, 1] - img1_offsety, 0, center[0])
+
+            class_info1 = box1[:, 4:5] if box1.shape[-1] > 4 else None
+            keypoints1 = None
+            if box1.shape[-1] > 5:
+                keypoints1 = box1[:, 5:].reshape(box1.shape[0], -1, 2)
+                invisible_keypoints1 = np.less_equal(keypoints1, 0).astype(np.bool)
+                keypoints1[:, :, 0] = np.clip(keypoints1[:, :, 0] - img1_offsetx + center[1], center[1], width)
+                keypoints1[:, :, 1] = np.clip(keypoints1[:, :, 1] - img1_offsety, 0, center[0])
+                keypoints1[invisible_keypoints1] = -1
+
+            else:
+                pass
+            concate_list1 = None
+            if keypoints1 is None:
+                # print('concate_list1',location1.reshape((box1.shape[0], 4)).shape,class_info1.shape)
+                concate_list1 = [location1.reshape((box1.shape[0], 4)), class_info1]
+            else:
+                concate_list1 = [location1.reshape((box1.shape[0], 4)), class_info1, keypoints1.reshape((box1.shape[0], -1))]
+            box1 = np.concatenate(concate_list1, axis=-1)
+            concate_list_all.append(box1)
+
+        box2 = self.memory_cache[other_idxes[1]][spec].copy() if  self.memory_cache[other_idxes[1]][spec] is not None else None
+        if box2 is None or len(box2) == 0:
             pass
-
-        box2 = np.concatenate([location2.reshape((box2.shape[0], 4)), class_info2, keypoints2.reshape((box2.shape[0], -1))], axis=-1)
-
-        box3 = self.memory_cache[other_idxes[2]][spec].copy()
-        location3 = box3[:, :4].reshape(box3.shape[0], 2, 2)
-        location3[:, :, 0] = np.clip(location3[:, :, 0] - img3_offsetx + center[1], center[1], width)
-        location3[:, :, 1] = np.clip(location3[:, :, 1] - img3_offsety + center[0], center[0], height)
-
-        class_info3 = box3[:, 4:5] if box3.shape[-1] > 4 else None
-        keypoints3 = None
-        if box3.shape[-1] > 5:
-            keypoints3 = box3[:, 5:].reshape(box3.shape[0], -1, 2)
-            invisible_keypoints3 = np.less_equal(keypoints3, 0).astype(np.bool)
-            keypoints3[:, :, 0] = np.clip(keypoints3[:, :, 0] - img3_offsetx + center[1], center[1], width)
-            keypoints3[:, :, 1] = np.clip(keypoints3[:, :, 1] - img3_offsety + center[0], center[0], height)
-            keypoints3[invisible_keypoints3] = -1
-
-
         else:
+            location2 = box2[:, :4].reshape(box2.shape[0], 2, 2)
+            location2[:, :, 0] = np.clip(location2[:, :, 0] - img2_offsetx, 0, center[1])
+            location2[:, :, 1] = np.clip(location2[:, :, 1] - img2_offsety + center[0], center[0], height)
+
+            class_info2 = box2[:, 4:5] if box2.shape[-1] > 4 else None
+            keypoints2 = None
+            if box2.shape[-1] > 5:
+                keypoints2 = box2[:, 5:].reshape(box2.shape[0], -1, 2)
+                invisible_keypoints2 = np.less_equal(keypoints2, 0).astype(np.bool)
+                keypoints2[:, :, 0] = np.clip(keypoints2[:, :, 0] - img2_offsetx, 0, center[1])
+                keypoints2[:, :, 1] = np.clip(keypoints2[:, :, 1] - img2_offsety + center[0], center[0], height)
+                keypoints2[invisible_keypoints2] = -1
+
+
+            else:
+                pass
+            concate_list2 = None
+            if keypoints2 is None:
+                # print('concate_list2',location2.reshape((box2.shape[0], 4)).shape,class_info2.shape)
+                concate_list2 = [location2.reshape((box2.shape[0], 4)), class_info2]
+            else:
+                concate_list2 = [location2.reshape((box2.shape[0], 4)), class_info2, keypoints2.reshape((box2.shape[0], -1))]
+
+            box2 = np.concatenate(concate_list2, axis=-1)
+            concate_list_all.append(box2)
+
+        box3 = self.memory_cache[other_idxes[2]][spec].copy() if  self.memory_cache[other_idxes[2]][spec] is not None else None
+        if box3 is None or len(box3) == 0:
             pass
+        else:
+            location3 = box3[:, :4].reshape(box3.shape[0], 2, 2)
+            location3[:, :, 0] = np.clip(location3[:, :, 0] - img3_offsetx + center[1], center[1], width)
+            location3[:, :, 1] = np.clip(location3[:, :, 1] - img3_offsety + center[0], center[0], height)
 
-        box3 = np.concatenate([location3.reshape((box3.shape[0], 4)), class_info3, keypoints3.reshape((box3.shape[0], -1))], axis=-1)
+            class_info3 = box3[:, 4:5] if box3.shape[-1] > 4 else None
+            keypoints3 = None
+            if box3.shape[-1] > 5:
+                keypoints3 = box3[:, 5:].reshape(box3.shape[0], -1, 2)
+                invisible_keypoints3 = np.less_equal(keypoints3, 0).astype(np.bool)
+                keypoints3[:, :, 0] = np.clip(keypoints3[:, :, 0] - img3_offsetx + center[1], center[1], width)
+                keypoints3[:, :, 1] = np.clip(keypoints3[:, :, 1] - img3_offsety + center[0], center[0], height)
+                keypoints3[invisible_keypoints3] = -1
 
-        trans_boxes = np.concatenate([box0, box1, box2, box3], axis=0)
+
+            else:
+                pass
+            concate_list3 = None
+            if keypoints3 is None:
+                # print('concate_list3',location3.reshape((box3.shape[0], 4)).shape,class_info3.shape)
+                concate_list3 = [location3.reshape((box3.shape[0], 4)), class_info3]
+            else:
+                concate_list3 = [location3.reshape((box3.shape[0], 4)), class_info3, keypoints3.reshape((box3.shape[0], -1))]
+            box3 = np.concatenate(concate_list3, axis=-1)
+            concate_list_all.append(box3)
+
+        trans_boxes = np.concatenate(concate_list_all, axis=0)
 
         hw = clip(trans_boxes[:, :4][..., 2:] - trans_boxes[:, :4][..., :2], 0.0, None)
         area = hw[..., 0] * hw[..., 1]
@@ -2237,4 +2362,4 @@ class ImageMosaic(VisionTransform):
             img3_offsetx = img3.shape[1] if img3.shape[1] - 1 < (width - center[1]) else np.random.choice(np.arange(0, img3.shape[1] - (width - center[1]), 1))
             img3_offsety = img3.shape[0] if img3.shape[0] - 1 < (height - center[0]) else np.random.choice(np.arange(0, img3.shape[0] - (height - center[0]), 1))
 
-        return (keep, other_idxes, center, img1_offsetx, img1_offsety, img2_offsetx, img2_offsety, img3_offsetx, img3_offsety)
\ No newline at end of file
+        return keep, other_idxes, center, img1_offsetx, img1_offsety, img2_offsetx, img2_offsety, img3_offsetx, img3_offsety
\ No newline at end of file
