@@ -670,12 +670,12 @@ class _ConvNd(Layer):
 
                     if self.transposed:
                         # filter_height, filter_width,  out_channels in_channels,
-                        self.weight =Parameter(tf.random.normal(shape=[*self.kernel_size, int(channel_multiplier), self.input_filters], mean=0,  stddev=1) * 0.02, trainable=True, name='weight')
+                        self.weight =Parameter(tf.random.normal(shape=[*self.kernel_size,  self.input_filters // self.groups, self.input_filters], mean=0,  stddev=1) * 0.02, trainable=True, name='weight')
 
                     else:
 
                         # [filter_height, filter_width, in_channels, out_channels]`
-                        self.weight =Parameter( data=tf.random.normal(shape=[*self.kernel_size, self.input_filters, int(channel_multiplier)], mean=0,    stddev=1) * 0.02, trainable=True, name='weight')
+                        self.weight =Parameter( data=tf.random.normal(shape=[*self.kernel_size, self.input_filters,  self.input_filters // self.groups], mean=0,    stddev=1) * 0.02, trainable=True, name='weight')
 
                         if self.separable:
                             pointwise_kernel_size = (1,) * len(self.kernel_size)
