@@ -11,7 +11,7 @@ if get_backend()=='pytorch':
     from . import pytorch_senet as senet
     from . import pytorch_densenet as densenet
     from . import pytorch_efficientnet as efficientnet
-    from . import pytorch_efficientnetv2 as efficientnet_v2
+    #from . import pytorch_efficientnetv2 as efficientnet_v2
     from . import pytorch_mobilenet as mobilenet
     from . import pytorch_deeplab as deeplab
     from . import pytorch_arcfacenet as arcfacenet
@@ -30,6 +30,7 @@ elif get_backend()=='tensorflow':
     from . import tensorflow_mobilenet as mobilenet
     from . import tensorflow_deeplab as deeplab
     from . import tensorflow_mtcnn as mtcnn
+    from . import tensorflow_arcfacenet as arcfacenet
 
 #__all__ = ['vgg','resnet','densenet','efficientnet','mobilenet','gan','deeplab','arcfacenet','mtcnn','rfbnet','ssd','yolo']
 
