@@ -24,7 +24,8 @@ from torch.nn import init
 from torch.nn.parameter import Parameter
 
 from trident.backend.common import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, ModuleList, fix_layer, load,get_device
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, ModuleList, fix_layer, load, \
+    get_device
 from trident.backend.pytorch_ops import *
 from trident.data.bbox_common import xywh2xyxy, xyxy2xywh
 from trident.data.image_common import *
@@ -36,7 +37,8 @@ from trident.layers.pytorch_normalizations import get_normalization
 from trident.layers.pytorch_pooling import *
 from trident.optims.pytorch_trainer import *
 from trident.models.pytorch_ssd import *
-from trident.data.vision_transforms import Resize,Normalize
+from trident.data.vision_transforms import Resize, Normalize
+
 __all__ = ['RfbNet', 'generate_priors']
 
 _session = get_session()
@@ -167,28 +169,37 @@ def basic_rfb(num_filters, scale=0.1):
     return ShortCut2d(Sequential(
         ShortCut2d(
             Sequential(
-                Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=False, use_bias=False, activation=None,
+                Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=False, use_bias=False,
+                             activation=None,
                              normalization='batch'),
-                Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, padding=(1, 1), use_bias=False, activation='relu',
+                Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, padding=(1, 1), use_bias=False,
+                             activation='relu',
                              normalization='batch'),
-                Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(2, 2), use_bias=False, dilation=2,
+                Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(2, 2), use_bias=False,
+                             dilation=2,
                              activation=None, normalization='batch'), name='branch1'),
 
             Sequential(
-                Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=False, use_bias=False, activation=None,
+                Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=False, use_bias=False,
+                             activation=None,
                              normalization='batch'),
-                Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, padding=(1, 1), use_bias=False, activation='relu',
+                Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, padding=(1, 1), use_bias=False,
+                             activation='relu',
                              normalization='batch'),
-                Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(3, 3), use_bias=False, dilation=3,
+                Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(3, 3), use_bias=False,
+                             dilation=3,
                              activation=None, normalization='batch'), name='branch2'),
             Sequential(
-                Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=False, use_bias=False, activation=None,
+                Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=False, use_bias=False,
+                             activation=None,
                              normalization='batch'),
-                Conv2d_Block((3, 3), depth_multiplier=1.5, strides=1, groups=1, padding=(1, 1), use_bias=False, activation='relu',
+                Conv2d_Block((3, 3), depth_multiplier=1.5, strides=1, groups=1, padding=(1, 1), use_bias=False,
+                             activation='relu',
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=1.33, strides=1, groups=1, padding=(1, 1), use_bias=False,
                              activation='relu', normalization='batch'),
-                Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(5, 5), use_bias=False, dilation=5,
+                Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(5, 5), use_bias=False,
+                             dilation=5,
                              activation=None, normalization='batch'), name='branch3')
             , mode='concate'),
 
@@ -226,9 +237,9 @@ def tiny_mobile_rfbnet(filter_base=16, num_classes=2):
                       conv_dw(filter_base * 16, 1))
 
 
-
 class RFBnet(Layer):
-    def __init__(self, *args, base_filters=16, num_classes=2, num_regressors=4,detection_threshold=0.7, nms_threshold=0.3, center_variance=0.1, size_variance=0.2,
+    def __init__(self, *args, base_filters=16, num_classes=2, num_regressors=4, detection_threshold=0.7,
+                 nms_threshold=0.3, center_variance=0.1, size_variance=0.2,
                  name='tiny_mobile_rfbnet', **kwargs):
         """
 
@@ -243,7 +254,7 @@ class RFBnet(Layer):
         self.backbond2 = Sequential(*backbond[8:11], name='backbond2')
         self.backbond3 = Sequential(*backbond[11:13], name='backbond3')
         self.detection_threshold = detection_threshold
-        self.nms_threshold =nms_threshold
+        self.nms_threshold = nms_threshold
         self.variance = (center_variance, size_variance)
         self.softmax = SoftMax(axis=-1)
 
@@ -255,8 +266,10 @@ class RFBnet(Layer):
         self.extra = Sequential(Conv2d((1, 1), num_filters=64, strides=1, activation='relu', use_bias=True),
                                 DepthwiseConv2d((3, 3), depth_multiplier=1, strides=2, auto_pad=True, activation='relu',
                                                 use_bias=True),
-                                Conv2d((1, 1), num_filters=256, strides=1, activation=None, use_bias=True), Relu(), name='extra')
-        self.regression_headers = ModuleList([Sequential(
+                                Conv2d((1, 1), num_filters=256, strides=1, activation=None, use_bias=True), Relu(),
+                                name='extra')
+        self.regression_headers = ModuleList([
+            Sequential(
             DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
             Conv2d((1, 1), num_filters=3 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
@@ -268,7 +281,8 @@ class RFBnet(Layer):
                 DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
                 Conv2d((1, 1), num_filters=2 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
-            Conv2d((3, 3), num_filters=3 * self.num_regressors, strides=1, auto_pad=True, activation=None), ], name='regression_headers')
+            Conv2d((3, 3), num_filters=3 * self.num_regressors, strides=1, auto_pad=True, activation=None) ],
+            name='regression_headers')
         self.classification_headers = ModuleList([
             Sequential(
                 DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
@@ -283,7 +297,7 @@ class RFBnet(Layer):
                 Conv2d((1, 1), num_filters=2 * self.num_classes, strides=1, activation=None, use_bias=True)),
 
             Conv2d((3, 3), num_filters=3 * self.num_classes, strides=1, auto_pad=True, activation=None,
-                   use_bias=True), ], name='classification_headers')
+                   use_bias=True) ], name='classification_headers')
 
     def define_img_size(self, size=640):
         global image_size, feature_map_w_h_list, priors
@@ -302,7 +316,7 @@ class RFBnet(Layer):
             for k in range(0, len(feature_map_w_h_list[i])):
                 item_list.append(image_size[i] / feature_map_w_h_list[i][k])
             shrinkage_list.append(item_list)
-        self.register_buffer("priors",generate_priors(feature_map_w_h_list, shrinkage_list, image_size, min_boxes))
+        self.register_buffer("priors", generate_priors(feature_map_w_h_list, shrinkage_list, image_size, min_boxes))
 
     def compute_header(self, i, x):
         confidence = self.classification_headers[i](x)
@@ -310,7 +324,7 @@ class RFBnet(Layer):
         confidence = confidence.view(confidence.size(0), -1, self.num_classes)
 
         if 'softmax' not in self.__dict__:
-            self.softmax=SoftMax(-1)
+            self.softmax = SoftMax(-1)
         confidence = self.softmax(confidence)
 
         location = self.regression_headers[i](x)
@@ -392,7 +406,7 @@ class RFBnet(Layer):
         boxes = boxes
         confidences = confidences
         if detection_threshold is not None:
-            self.detection_threshold=detection_threshold
+            self.detection_threshold = detection_threshold
         picked_box_probs = []
         picked_labels = []
         for class_index in range(1, confidences.shape[1]):
@@ -416,7 +430,8 @@ class RFBnet(Layer):
         picked_box_probs[:, 1] *= height
         picked_box_probs[:, 2] *= width
         picked_box_probs[:, 3] *= height
-        return to_numpy(picked_box_probs[:, :4]).astype(np.int32), np.array(picked_labels), to_numpy(picked_box_probs[:, 4])
+        return to_numpy(picked_box_probs[:, :4]).astype(np.int32), np.array(picked_labels), to_numpy(
+            picked_box_probs[:, 4])
 
     def rerec(self, box, img_shape):
         """Convert box to square."""
@@ -464,7 +479,6 @@ class RFBnet(Layer):
         confidences = torch.cat(confidences, 1)
         locations = torch.cat(locations, 1)
 
-
         if self.training:
             result = OrderedDict()
             result['confidences'] = confidences
@@ -486,11 +500,13 @@ def RfbNet(include_top=True,
         input_shape = tuple(input_shape)
     else:
         input_shape = (3, 480, 640)
-    if num_classes != 2 or  num_regressors != 4:
-        pretrained=False
-    rfbnet = SsdDetectionModel(input_shape=input_shape, output=RFBnet(base_filters=base_filters, num_classes=num_classes, num_regressors=num_regressors))
-    rfbnet.detection_threshold=0.7
-    rfbnet.nms_threshold=0.7
+    if num_classes != 2 or num_regressors != 4:
+        pretrained = False
+    rfbnet = SsdDetectionModel(input_shape=input_shape,
+                               output=RFBnet(base_filters=base_filters, num_classes=num_classes,
+                                             num_regressors=num_regressors))
+    rfbnet.detection_threshold = 0.7
+    rfbnet.nms_threshold = 0.7
     rfbnet.palette[0] = (128, 255, 128)
     rfbnet.palette[1] = (128, 255, 128)
     rfbnet.preprocess_flow = [
@@ -498,24 +514,16 @@ def RfbNet(include_top=True,
         Normalize(127.5, 127.5)
     ]
 
-
-
     if pretrained == True:
         download_model_from_google_drive('1T_0VYOHaxoyuG1fAxY-6g0C7pfXiujns', dirname, 'version-RFB-640.pth')
-        recovery_model =fix_layer( load(os.path.join(dirname, 'version-RFB-640.pth')))
+        recovery_model = fix_layer(load(os.path.join(dirname, 'version-RFB-640.pth')))
         recovery_model.softmax = SoftMax(axis=-1)
-        priors=recovery_model.priors.clone()
+        priors = recovery_model.priors.clone()
         recovery_model.__delattr__("priors")
-        recovery_model.register_buffer("priors",priors)
+        recovery_model.register_buffer("priors", priors)
         recovery_model.name = 'rfb640'
         recovery_model.eval()
         recovery_model.to(_device)
-        rfbnet.model=recovery_model
+        rfbnet.model = recovery_model
     return rfbnet
 
-
-
-
-
-
-
