@@ -748,7 +748,7 @@ class SsdDetectionModel(ImageDetectionModel):
         overlap_area = self.area_of(overlap_left_top, overlap_right_bottom)
         area0 = self.area_of(boxes0[..., :2], boxes0[..., 2:])
         area1 = self.area_of(boxes1[..., :2], boxes1[..., 2:])
-        return overlap_area / (area0 + area1 - overlap_area + eps)
+        return overlap_area / (area0 + area1 - overlap_area + eps),overlap_area, (area0 + area1 - overlap_area + eps)
 
     def hard_nms(self, box_scores, nms_threshold, top_k=-1, candidate_size=200):
         """
@@ -882,20 +882,20 @@ class SsdDetectionModel(ImageDetectionModel):
                     self.palette = generate_palette(confidence.shape[-1])
 
                 probs = 1 - confidence[:, 0]
-                label = argmax(confidence[:, 1:], -1) + 1
+                label = argmax(confidence[:, :], -1)
 
                 # mask = label > 0
                 # probs = probs[mask]
                 # label = label[mask]
                 # boxes = boxes[mask, :]
-                print(probs.max())
+                #print(probs.max())
                 mask = probs > self.detection_threshold
                 probs = probs[mask]
                 label = label[mask]
                 boxes = boxes[mask, :]
 
                 if boxes is not None and len(boxes) > 0:
-                    box_probs = concate([boxes.float(), label.reshape(-1, 1).float(), probs.reshape(-1, 1).float()], axis=1)
+                    box_probs = to_numpy(concate([boxes.float(), label.reshape(-1, 1).float(), probs.reshape(-1, 1).float()], axis=1))
                     if len(boxes) > 1:
                         box_probs, keep = self.hard_nms(box_probs, nms_threshold=self.nms_threshold, top_k=-1, )
 
@@ -924,12 +924,16 @@ class SsdDetectionModel(ImageDetectionModel):
                 labels = np.expand_dims(labels, 0)
             print(img, time.time() - start_time)
             pillow_img = array2image(rgb_image.copy())
-            for m in range(len(boxes)):
-                this_box = boxes[m]
-                this_label = labels[m]
-                thiscolor = tuple([int(c) for c in self.palette[int(this_label) - 1][:3]])
-                print(img, self.class_names[int(this_label) - 1], this_box, probs[m])
-                pillow_img = plot_bbox(this_box, pillow_img, thiscolor, self.class_names[int(this_label) - 1] if self.class_names is not None else '', line_thickness=2)
+
+            print(boxes,labels,flush=True)
+            if len(boxes)>0:
+                for m in range(len(boxes)):
+                    this_box = boxes[m]
+                    this_label = labels[m]
+                    if int(this_label) >0:
+                        thiscolor = tuple([int(c) for c in self.palette[int(this_label) - 1][:3]])
+                        print(img, self.class_names[int(this_label) - 1], this_box, probs[m],flush=True)
+                        pillow_img = plot_bbox(this_box, pillow_img, thiscolor, self.class_names[int(this_label) - 1] if self.class_names is not None else '', line_thickness=2)
             rgb_image = np.array(pillow_img.copy())
 
         return rgb_image, boxes, labels, probs
