@@ -2,6 +2,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import numbers
 import sys
 from typing import Callable, Any
 
@@ -48,7 +49,7 @@ class _ClassificationLoss(Loss):
     """Calculate loss for  complex classification task."""
 
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
-                 ohem_ratio=3.5, binding_dataset_symbol=None,
+                 ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None,
                  name=None, **kwargs):
         """
 
@@ -82,7 +83,7 @@ class _ClassificationLoss(Loss):
             label_smooth (bool):If True, mean we will apply label-smoothing in loss calculation.
 
         """
-        super(_ClassificationLoss, self).__init__(reduction=reduction, sample_weight=sample_weight, axis=axis, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+        super(_ClassificationLoss, self).__init__(reduction=reduction, sample_weight=sample_weight, axis=axis, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,  input_names=input_names, output_names=output_names,name=name)
         self.need_target_onehot = False
         self.is_multiselection = False
         self.is_target_onehot = False
@@ -199,7 +200,7 @@ class _ClassificationLoss(Loss):
         else:
             pass
 
-
+        output_exp = exp(output)
         if ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_min(output) >= 0 :
             self.is_logsoftmax = False
             if reduce_max(output) > 1:
@@ -220,16 +221,15 @@ class _ClassificationLoss(Loss):
 
             output = clip(output, min=1e-7, max=1)
 
-        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_max(output) <=0):
+        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
             self.is_logsoftmax = True
             self.from_logits = True
-            output = clip(output,min=-7.0,max=-1e-8)
             if self.auto_balance and self.label_statistics is not None:
                 if int_shape(output)[1] == len(self.label_statistics):
                     new_shp = [1] * len(int_shape(output))
                     new_shp[1] = len(self.label_statistics)
-                    output = output - to_tensor(np.reshape(np.log(clip(self.label_statistics.copy(), min=1e-7)), tuple(new_shp)),requires_grad=False)
-
+                    output = output - to_tensor(np.reshape(np.log(clip(self.label_statistics.copy(), min=1e-7)), tuple(new_shp)))
+            output = clip(output, max=- 1e-7)
         else:
             self.is_logsoftmax = False
             self.from_logits = False
@@ -251,12 +251,13 @@ class _ClassificationLoss(Loss):
         if target.dtype == str2dtype('long'):
             self.is_target_onehot = False
         elif target.dtype != str2dtype('long') and (target.min() >= 0 and target.max() <= 1 and abs(target.sum(self.axis)-1).mean() < 1e-4):
-            target = clip(target, min=1e-7, max=1 )
+            target = clip(target, min=1e-7, max=1- 1e-7)
             self.is_target_onehot = True
 
         # need target onehot but currently not
         if target.dtype == torch.long and self.need_target_onehot == True and self.is_target_onehot == False:
             target = make_onehot(target, num_classes=self.num_classes, axis=self.axis).to(get_device())
+
             target.require_grads = False
             self.is_target_onehot = True
             if self.label_smooth:
@@ -342,7 +343,7 @@ class _ClassificationLoss(Loss):
 class _PairwiseLoss(Loss):
     """Calculate loss for  complex classification task."""
 
-    def __init__(self, axis=None, sample_weight=None, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name=None, **kwargs):
+    def __init__(self, axis=None, sample_weight=None, reduction='mean', enable_ohem=False, ohem_ratio=3.5, input_names=None, output_names=None,name=None, **kwargs):
         """
 
         Args:
@@ -375,7 +376,7 @@ class _PairwiseLoss(Loss):
             label_smooth (bool):If True, mean we will apply label-smoothing in loss calculation.
 
         """
-        super(_PairwiseLoss, self).__init__(reduction=reduction, axis=axis, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+        super(_PairwiseLoss, self).__init__(reduction=reduction, axis=axis, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,  input_names=input_names, output_names=output_names, name=name)
         self.sample_weight = sample_weight
 
         # initilize weight
@@ -410,6 +411,8 @@ class _PairwiseLoss(Loss):
         # output, target = self.flatten_check(output, target)
         if self.sample_weight is None or len(self.sample_weight) != int_shape(output)[0]:
             self.sample_weight = ones((int_shape(output)[0]), dtype=output.dtype)
+        if isinstance(target,numbers.Number):
+            target=ones_like(output)*target
 
         if output.shape == target.shape:
             if self.enable_ohem:
@@ -564,7 +567,7 @@ class CrossEntropyLoss(_ClassificationLoss):
     """
 
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
-                 ohem_ratio=3.5, binding_dataset_symbol=None, name='CrossEntropyLoss'):
+                 ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None, name='CrossEntropyLoss'):
         """
 
         Args:
@@ -578,7 +581,7 @@ class CrossEntropyLoss(_ClassificationLoss):
             name (stringf):
         """
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol,  input_names=input_names, output_names=output_names, name=name)
         self._built = True
         self.need_target_onehot = False
 
@@ -729,9 +732,9 @@ class NLLLoss(_ClassificationLoss):
     """
 
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
-                 ohem_ratio=3.5, binding_dataset_symbol=None, name='NllLoss'):
+                 ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None, name='NllLoss'):
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol ,  input_names=input_names, output_names=output_names, name=name)
         self.need_target_onehot = False
         self._built = True
 
@@ -811,9 +814,9 @@ class F1ScoreLoss(_ClassificationLoss):
     """
 
     def __init__(self, beta=1, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean',
-                 enable_ohem=False, ohem_ratio=3.5, binding_dataset_symbol=None, name='CrossEntropyLoss'):
+                 enable_ohem=False, ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None, name='CrossEntropyLoss'):
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol ,  input_names=input_names, output_names=output_names, name=name)
         self.beta = beta
 
         self.need_target_onehot = True
@@ -873,9 +876,9 @@ class FocalLoss(_ClassificationLoss):
     """
 
     def __init__(self, alpha=0.5, gamma=2, normalized=False, threshold=None, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None,
-                 label_smooth=False, reduction='mean', enable_ohem=False, ohem_ratio=3.5, binding_dataset_symbol=None, name='FocalLoss'):
+                 label_smooth=False, reduction='mean', enable_ohem=False, ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None, name='FocalLoss'):
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=None, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=None ,  input_names=input_names, output_names=output_names, name=name)
         self.alpha = alpha
         self.gamma = gamma
         self.threshold = threshold
@@ -952,9 +955,9 @@ class FocalLoss(_ClassificationLoss):
 
 class BCELoss(_ClassificationLoss):
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
-                 ohem_ratio=3.5, binding_dataset_symbol=None, name='BCELoss'):
+                 ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None, name='BCELoss'):
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol ,  input_names=input_names, output_names=output_names, name=name)
         self._built = True
         self.num_classes = None
         self.is_logsoftmax = False
@@ -1021,7 +1024,7 @@ class DiceLoss(_ClassificationLoss):
     """
 
     def __init__(self, smooth=1., axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean',
-                 enable_ohem=False, ohem_ratio=3.5, binding_dataset_symbol=None, name='DiceLoss'):
+                 enable_ohem=False, ohem_ratio=3.5, binding_dataset_symbol=None,  input_names=None, output_names=None,name='DiceLoss'):
         """
         Args:
             axis (int): the axis where the class label is.
@@ -1035,7 +1038,7 @@ class DiceLoss(_ClassificationLoss):
         """
 
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol ,  input_names=input_names, output_names=output_names, name=name)
         self.smooth = smooth
         self.is_logsoftmax = False
         self.need_target_onehot = True
@@ -1074,9 +1077,9 @@ class DiceLoss(_ClassificationLoss):
 
 class KLDivergenceLoss(_ClassificationLoss):
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
-                 ohem_ratio=3.5, binding_dataset_symbol=None, name='KLDivergenceLoss'):
+                 ohem_ratio=3.5, binding_dataset_symbol=None, input_names=None, output_names=None, name='KLDivergenceLoss'):
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol, name=name)
+                         label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, binding_dataset_symbol=binding_dataset_symbol ,  input_names=input_names, output_names=output_names, name=name)
         self._built = True
         self.num_classes = 1
         self.is_logsoftmax = False
@@ -1106,8 +1109,8 @@ class L1Loss(_PairwiseLoss):
      See :class:`~torch.nn.L1Loss` for details.
      """
 
-    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name='L1Loss'):
-        super(L1Loss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, input_names=None, output_names=None,name='L1Loss'):
+        super(L1Loss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio ,  input_names=input_names, output_names=output_names, name=name)
         self.name = name
         self.reduction = reduction
 
@@ -1134,8 +1137,8 @@ class L2Loss(_PairwiseLoss):
         See :class:`~torch.nn.MSELoss` for details.
         """
 
-    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name='MSELoss'):
-        super(L2Loss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5,  input_names=None, output_names=None ,name='MSELoss'):
+        super(L2Loss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,   input_names=input_names, output_names=output_names,name=name)
         self.name = name
         self.reduction = reduction
 
@@ -1161,8 +1164,8 @@ class SmoothL1Loss(_PairwiseLoss):
     See :class:`~torch.nn.SmoothL1Loss` for details.
     """
 
-    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name='SmoothL1Loss'):
-        super(SmoothL1Loss, self).__init__(enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, reduction=reduction, name=name)
+    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5,  input_names=None, output_names=None ,  name='SmoothL1Loss'):
+        super(SmoothL1Loss, self).__init__(enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, reduction=reduction,input_names=input_names, output_names=output_names, name=name)
         self.name = name
         self.reduction = reduction
         self.huber_delta = 0.5
@@ -1190,8 +1193,8 @@ class MSELoss(_PairwiseLoss):
         See :class:`~torch.nn.MSELoss` for details.
         """
 
-    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name='MSELoss'):
-        super(MSELoss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5,  input_names=None, output_names=None,name='MSELoss'):
+        super(MSELoss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio  ,  input_names=input_names, output_names=output_names, name=name)
         self.name = name
         self.reduction = reduction
 
@@ -1211,8 +1214,8 @@ class MSELoss(_PairwiseLoss):
 
 
 class WingLoss(_PairwiseLoss):
-    def __init__(self, omega=10, epsilon=2, name='WingLoss'):
-        super(WingLoss, self).__init__()
+    def __init__(self, omega=10, epsilon=2,  input_names=None, output_names=None,name='WingLoss'):
+        super(WingLoss, self).__init__(  input_names=input_names, output_names=output_names, name=name)
         self.name = name
         self.omega = omega
         self.epsilon = epsilon
@@ -1241,8 +1244,8 @@ class WingLoss(_PairwiseLoss):
 
 
 class AdaptiveWingLoss(_PairwiseLoss):
-    def __init__(self, omega=14, theta=0.5, epsilon=1, alpha=2.1, name='AdaptiveWingLoss'):
-        super(AdaptiveWingLoss, self).__init__()
+    def __init__(self, omega=14, theta=0.5, epsilon=1, alpha=2.1,  input_names=None, output_names=None,name='AdaptiveWingLoss'):
+        super(AdaptiveWingLoss, self).__init__(  input_names=input_names, output_names=output_names, name=name)
         self.name = name
         self.omega = omega
         self.theta = theta
@@ -1271,8 +1274,8 @@ class AdaptiveWingLoss(_PairwiseLoss):
 
 
 class ExponentialLoss(_PairwiseLoss):
-    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name='ExponentialLoss'):
-        super(ExponentialLoss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5,  input_names=None, output_names=None,name='ExponentialLoss'):
+        super(ExponentialLoss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, input_names=input_names, output_names=output_names,name=name)
         self.name = name
         self.reduction = reduction
 
@@ -1297,8 +1300,8 @@ class ExponentialLoss(_PairwiseLoss):
 
 
 class ItakuraSaitoLoss(_PairwiseLoss):
-    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5, name='ItakuraSaitoLoss'):
-        super(ItakuraSaitoLoss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+    def __init__(self, reduction='mean', enable_ohem=False, ohem_ratio=3.5,  input_names=None, output_names=None,name='ItakuraSaitoLoss'):
+        super(ItakuraSaitoLoss, self).__init__(reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,input_names=input_names, output_names=output_names ,name=name)
         self.name = name
         self.reduction = reduction
 
@@ -1326,8 +1329,8 @@ class ItakuraSaitoLoss(_PairwiseLoss):
 
 
 class CosineSimilarityLoss(_PairwiseLoss):
-    def __init__(self, enable_ohem=False, ohem_ratio=3.5, name='CosineSimilarityLoss'):
-        super(CosineSimilarityLoss, self).__init__(enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
+    def __init__(self, enable_ohem=False, ohem_ratio=3.5,  input_names=None, output_names=None,name='CosineSimilarityLoss'):
+        super(CosineSimilarityLoss, self).__init__(enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,input_names=input_names, output_names=output_names, name=name)
 
     def calculate_loss(self, output, target, **kwargs):
         """
@@ -1428,7 +1431,7 @@ def ssim(X, Y, window, data_range: float, use_padding: bool = False):
 
 
 class MS_SSIMLoss(_PairwiseLoss):
-    def __init__(self, window_size=11, window_sigma=1.5, data_range=255., channel=3, use_padding=False, weights=None, levels=4, eps=1e-8):
+    def __init__(self, window_size=11, window_sigma=1.5, data_range=255., channel=3, use_padding=False, weights=None, levels=4, eps=1e-8,input_names=None, output_names=None,name='ms_ssim'):
         super(MS_SSIMLoss, self).__init__()
 
         self.window_size = window_size
@@ -1498,7 +1501,7 @@ class MS_SSIMLoss(_PairwiseLoss):
 
 class IoULoss(_ClassificationLoss):
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
-                 ohem_ratio=3.5, name='lou_loss'):
+                 ohem_ratio=3.5,  input_names=None, output_names=None,name='lou_loss'):
         super(IoULoss, self).__init__(raxis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
                                       label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
         self.need_target_onehot = False
@@ -1527,7 +1530,7 @@ class IoULoss(_ClassificationLoss):
 
 
 class SoftIoULoss(Loss):
-    def __init__(self, n_classes, reduction="mean", reduced=False):
+    def __init__(self, n_classes, reduction="mean", reduced=False, input_names=None, output_names=None,name='softiou'):
         super(SoftIoULoss, self).__init__()
         self.n_classes = n_classes
         self.reduction = reduction
@@ -1913,18 +1916,19 @@ class CenterLoss(_Loss):
         init.kaiming_uniform_(self.centers, a=sqrt(5))
         self.to(_device)
 
-    def forward(self, output, target) -> 'loss':
+    def forward(self, features, target) -> 'loss':
         """
         Args:
-            output: feature matrix with shape (batch_size, feat_dim).
+            features: feature matrix with shape (batch_size, feat_dim).
             target: ground truth labels with shape (num_classes).
 
         """
-        assert output.size(0) == target.size(0), "features.size(0) is not equal to labels.size(0)"
-        batch_size = output.size(0)
-        distmat = torch.pow(output, 2).sum(dim=1, keepdim=True).expand(batch_size, self.num_classes) + torch.pow(
+
+        batch_size = features.size(0)
+        self.centers.to(features.dtype).to(features.device)
+        distmat = torch.pow(features, 2).sum(dim=1, keepdim=True).expand(batch_size, self.num_classes) + torch.pow(
             self.centers, 2).sum(dim=1, keepdim=True).expand(self.num_classes, batch_size).t()
-        distmat.addmm_(1, -2, output, self.centers.t())
+        distmat.addmm_(1, -2, features, self.centers.t())
         classes = torch.arange(self.num_classes).long().to(_device)
 
         target = target.unsqueeze(1).expand(batch_size, self.num_classes)
