@@ -4,6 +4,7 @@ from __future__ import print_function
 
 import builtins
 import math
+import numbers
 import sys
 
 import numpy as np
@@ -413,7 +414,11 @@ class _PairwiseLoss(Loss):
         Returns:
 
         """
+        if isinstance(target, numbers.Number):
+            target = ones_like(output) * target
+
         output, target = self.flatten_check(output, target)
+
         if output.shape == target.shape:
             return output, target
 
